
#ifndef TSC_LIB_GNUC_HH
#define TSC_LIB_GNUC_HH

/* used for gettimeofday and usleep. */
#include <sys/time.h>
#include <unistd.h>
#include <stdio.h>

/*
  The time stamp counter is returned as a 64 bit counter. Since GNUC compilers
  have a 64 bit integral type, we define ULONGLONG, and we define a flag that
  this compiler supports the longer type. We also define a macro that casts
  such a type into a double, because of microsoft's stupidity (they do not
  have such a cast).
	*/
#define HAVE_ULONGLONG
typedef unsigned long long TSC_ULONGLONG;
#define ULL_TO_DOUBLE(ull) ((double)ull)

/*
  Again, because not all compilers use the same inline syntax, we have to map
  all possible values to our "standard" one.
*/
#define INLINE extern __inline__

/*
  Reads the 64bit time stamp counter. Since these machines are 32 bit
  ones, we have to read it in two halves.
  */
#define rdtsc(cnt) gnu_rdtsc(&(cnt))
INLINE void gnu_rdtsc(tsc_int64 * const cnt) 
{
#if 1
  asm volatile(".byte 0x0f, 0x31" : /* we use the older as so it does not
				       recognize the rdtsc instruction. */
	       "=a" ((cnt)->low),
	       "=d" ((cnt)->high) :
	       : "eax", "edx");
#else
  asm volatile("rdtsc" :
	       "=a" ((cnt)->low),
	       "=d" ((cnt)->high) :
	       : "eax", "edx");
#endif
}

/*
  These are private functions of the library. Use this #ifdef to mask them from the outside world. */
#ifdef TSC_PRIVATE_FUNCTIONS

/*
  Prints the ULONGLONG type into a character array.
*/
INLINE char *tsc_platformULLToStr(TSC_ULONGLONG counter, char* buffer)
{
  /* GNU supports the %llu flag. */
  sprintf(buffer, "%llu", counter);
  return buffer;
}

/*
  A milisecond timer. Note: we do not care what it times (e.g. milisecs since
  when) because	we only need it for a time difference between calls. Do not
  use it as an absolute timer.
*/
INLINE unsigned long tsc_msTime()
{
  struct timeval tv;
  if(gettimeofday(&tv, NULL) != 0) {
    return 0;
  }
  /* return the time in milisecs. Cut the lower microsecs. */
  return tv.tv_sec*1000 + tv.tv_usec/1000; 
}

/*
  Sleeps for msec miliseconds.
*/
INLINE void tsc_msSleep(unsigned long msec)
{
  /* usleep accept a time in usecs - multiply msec by 1000 to convert. */
  usleep(msec*1000);
}

#endif /* TSC_PRIVATE_FINCTIONS */

#endif
