#include <stdio.h> 
#include <stdlib.h> 
#include <process.h> 
#include <sys/qnx_glob.h> 
#include <signal.h>
#include <sys/types.h>
#include <sys/times.h>
#include <sys/wait.h>
#include <math.h>
#include "time_util.h"

#define WAIT_FOR_CHILDREN 1 /* if 1 then wait for sigchild after each fork */
#define WAIT 999
#define MAX_DATAPOINTS 1000
#define MILLION 1000000
#define SLICE_FRAC .1
#define FALSE 0
#define TRUE  1

void spawn_proc(char *filename, int iter, int warmup, char **arge);
void calc_stdev(int iter);
void calc_times(int iter);
void calc_stdev(int iter);

/* Used for measuring the overall time and the average */
struct timespec g_time_before, g_time_after;

/* Used to measure the average time of each slice */
struct timespec g_slice_datapoints[MAX_DATAPOINTS];

double g_avg_micro; /* average time for all samples in microseconds */
int g_iterations_per_slice;

int main(int argc, char **argv, char **arge)
{
  int iterations, warmup_iter = 0;

  if ((argc != 3) && (argc != 4)){
    fprintf(stderr, "Usage: %s <filename> <iterations> [warmup iterations]\n",
	    argv[0]);
    exit(1);
  }
  //init();
  iterations = atoi(argv[2]);
  if (argc == 4)
    warmup_iter = atoi(argv[3]);
          
  if (warmup_iter)
    spawn_proc(argv[1], warmup_iter, TRUE, arge);
  
  spawn_proc(argv[1], iterations, FALSE, arge);

  return 0;
}

void spawn_proc(char *filename, int iter, int warmup, char **arge)
{
  //struct timespec time_before, time_after, total_time, avg_time;
  int i, pid;
  static char *arg_list[10];
  int datapoints_count;
  int status;

  arg_list[0] = filename;
  arg_list[1] = NULL;
  
  datapoints_count = 0;
  g_iterations_per_slice = (int)(iter *SLICE_FRAC + 0.5);
  if (g_iterations_per_slice == 0) {
    g_iterations_per_slice = 1000; /* dummy value for the warmup iterations */
  }
  clock_gettime( CLOCK_REALTIME, &g_time_before);
  for (i=0; i<iter; i++){
    if ((i % g_iterations_per_slice) == 0){
      clock_gettime( CLOCK_REALTIME, &g_slice_datapoints[datapoints_count]);
      datapoints_count++;
    }
  	
    pid = qnx_spawn(  
        0,       /*  create a new process */ 
        NULL,    /* let it alloc buffer */ 
        5,  /* run on this node */ 
        -1,      /* inherit current priority */ 
        -1,      /* inherit current scheduler */ 
        0,       /* nothing fancy */ 
        filename, /* this is the command */ 
        arg_list,  /* the command and its args */ 
        arge,    /* pass along this environ   */ 
        NULL,    /* default: inherit all fd's */ 
        -1 
      ); 
    if( pid == -1 ) 
      { 
      perror( "qnx_spawn" ); 
      exit( EXIT_FAILURE ); 
      } 
    else { 
      wait( &status ); 
    }
  } 
  clock_gettime( CLOCK_REALTIME, &g_time_after);
  clock_gettime( CLOCK_REALTIME, &g_slice_datapoints[datapoints_count]);
  datapoints_count++;
  if (warmup) /* no need for calculations when warming up */
    return;
    
  calc_times(iter);
  calc_stdev(datapoints_count);   
}
 
void calc_times(int iter)
{
  struct timespec total_time;
 
  /* calculate the total and average time measured by gettimeofday */
  fprintf(stderr, "time after = %d:%d; time before = %ld:%ld\n", 
  		  g_time_after.tv_sec, g_time_after.tv_nsec, g_time_before.tv_sec, g_time_before.tv_nsec);
  
  total_time = t_minus(g_time_after, g_time_before);
  fprintf(stderr, "Total elaped time: %ld:%ld(Sec:MicroSec) ; ", total_time.tv_sec, total_time.tv_nsec/1000);
  g_avg_micro = ((total_time.tv_sec * MILLION) + total_time.tv_nsec/1000) / (iter * 1.0); 
  fprintf(stderr, "Average elapsed time: %3.3f(MicroSec)\n", g_avg_micro);
} 

void calc_stdev(int datapoints)
{  
  static double slice_avg[MAX_DATAPOINTS];
  double diff, slices_avg, sum, stdev;
  struct timespec slice_total_time;  
  int i;
  double slices_avg_sum = 0.;

  for (i=1; i<datapoints; i++){
    slice_total_time = t_minus(g_slice_datapoints[i], g_slice_datapoints[i-1]);
    slice_avg[i-1] = (slice_total_time.tv_sec * MILLION + slice_total_time.tv_nsec/1000) / (g_iterations_per_slice*1.0);
    slices_avg_sum += slice_avg[i-1];
    fprintf(stderr, "val = %3.3f\n", slice_avg[i-1]); 
  }

  slices_avg = slices_avg_sum / ((datapoints-1) * 1.0);
  fprintf(stderr, "Average of all slices is %3.3f\n", slices_avg);
  sum = 0;
  for (i=0; i<datapoints-1; i++){
    diff = slice_avg[i] - slices_avg;
    sum += diff*diff;
  }
  stdev = sqrt(sum / (datapoints - 1));
  fprintf(stderr, "(before) The STDDEV of %d slices is: %3.3f\n", datapoints-1, stdev);
  stdev *= sqrt(g_iterations_per_slice);
  
  fprintf(stderr, "The STDDEV of %d slices is: %3.3f\n", datapoints-1, stdev);
}


/* old code
  clock_t  total_user_time_with_children, total_kernel_time_with_children,
  	total_user_time, total_kernel_time;
   struct tms sliced_time_before, sliced_time_after;
 
  total_user_time = sliced_time_after.tms_utime - sliced_time_before.tms_utime ;
  total_kernel_time = sliced_time_after.tms_stime - sliced_time_before.tms_stime;

  total_user_time_with_children    = sliced_time_after.tms_cutime - sliced_time_before.tms_cutime ;
  total_kernel_time_with_children = sliced_time_after.tms_cstime - sliced_time_before.tms_cstime ;
  
  fprintf(stdout, "parent: user time %ld ; kernel time %ld\n", total_user_time, total_kernel_time);
  fprintf(stdout, "parent+children:  user time %ld ; kernel time %ld\n", total_user_time_with_children, 
  						total_kernel_time_with_children);
  			
  fprintf(stdout, "parent+children total user time(milliseconds) %ld\n", sliced_time_after.tms_cutime);
  fprintf(stdout, "parent+children total kernel time(milliseconds) %ld\n", sliced_time_after.tms_cstime);
  fflush(stdout);*/

