/******************************************************************
This program is used to check the time it takes to create a process
in Win NT.
Run it from the command line with the following arguments:
<filename> <iterations> [warmup iterations]
*******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

// if set, wait for the child proc to die before spawning a new one
#define WAIT_FOR_CHILDREN 1 
FILE *outfile;


void ProcRun(char *path, int iter, int warmup_iter = 0);
void TraslateTimeToMili(DWORD Time, DWORD &Min, DWORD &Mili);
void TraslateTimeToMicro(DWORD Time, DWORD &Min, DWORD &Micro);

void main(int argc, char **argv)
{
  //fprintf(stderr, "Running runproc...\n");

  if ((argc != 4) && (argc !=5)){
    fprintf(stderr, "Usage: %s <app filename> <iterations> <output filename>[warmup iterations]\n",
            argv[0]);
    exit(1);
  }
  
  outfile = fopen(argv[3], "a+");
  if (outfile == NULL) {
	fprintf(stderr, "Error: failed to open output file.\n");
	exit(1);
  }
   
  if (argc == 4)
	ProcRun(argv[1], atoi(argv[2]));
  else
	ProcRun(argv[1], atoi(argv[2]), atoi(argv[4]));

  fclose(outfile);
}

void ProcRun(char *path, int iter, int warmup_iter)
{
  STARTUPINFO si; 
  PROCESS_INFORMATION info;
  int ret_val;
  DWORD TimeBefore; 

  fprintf(stderr, "Running file %s, %d times\n", path, iter);

  if (warmup_iter == 0)
	TimeBefore = GetTickCount();

   // Set up members of STARTUPINFO structure. 
  ZeroMemory( &si, sizeof(STARTUPINFO) );
  si.cb = sizeof(STARTUPINFO); 
  si.lpDesktop = "WinSta0\\Default"; 
  si.dwFlags = STARTF_USESTDHANDLES;
  for (int i=0; i<iter + warmup_iter; i++) {
	  if ((warmup_iter != 0) && (i == warmup_iter)){
		fprintf(stderr, "Finished %d warmup iterations.\n", warmup_iter);
		TimeBefore = GetTickCount();
	  }
	ret_val = CreateProcess(
		path, 
		"",          // command line 
		NULL,          // process security attributes 
		NULL,          // primary thread security attributes
		TRUE,          // handles are inherited 
		DETACHED_PROCESS,             // creation flags
		NULL,          // use parent's environment 
		NULL,          // use parent's current directory 
		&si,		     // STARTUPINFO pointer 
		&info);  // receives PROCESS_INFORMATION 

	if (! ret_val) {
		fprintf(stderr, "Error: CreateProcess failed %d\n", GetLastError());
		exit(1);
	}

	if (WAIT_FOR_CHILDREN) {
		if (WaitForSingleObject (info.hProcess, INFINITE) == WAIT_FAILED){
			fprintf(stderr, "WaitForSingleObject failed :%d", GetLastError());
		}
	}
	CloseHandle(info.hProcess);
	CloseHandle(info.hThread);
  }
 
  DWORD TimeAfter = GetTickCount();
  DWORD TotalTime = TimeAfter - TimeBefore;
  DWORD TotalMin, TotalMili;
  TraslateTimeToMili(TotalTime, TotalMin, TotalMili);
//  fprintf(stdout, "Total time %d:%d (Sec:MiliSec) ; ", TotalMin, TotalMili);

  DWORD AvgTime = (TotalTime * 1000)/ iter;
  DWORD AvgMin, AvgMicro;
  TraslateTimeToMicro(AvgTime, AvgMin, AvgMicro);	

  fprintf(outfile, "Average time %d:%d(Sec:MicroSec) ; Total time %d:%d (Sec:MiliSec)\n",
		  AvgMin, AvgMicro);
  fflush(outfile);
			
}

void TraslateTimeToMili(DWORD Time, DWORD &Min, DWORD &Mili)
{
	Min = Time / 1000;
	Mili = Time % 1000;
}

void TraslateTimeToMicro(DWORD Time, DWORD &Min, DWORD &Micro)
{
	Min = Time / 1000000;
	Micro = Time % 1000000;
}