#include <windows.h>
#include <stdio.h>
#include <math.h>

#define MAX_ITERATIONS 10000000
#define MAX_THRESHOLDS 15
static double g_results[MAX_ITERATIONS];

int g_thresh[MAX_THRESHOLDS] = { 1, 2, 3, 4, 5, 7, 10, 50, 100, 1000, 10000 , 20000, 50000};
int g_thresh_count[MAX_THRESHOLDS];

void main(int argc, char **argv)
{	
	LARGE_INTEGER counter = {0, 0}, counter2 = {0, 0}, freq ={0 , 0};
	long dif, sum_difs = 0;
	double avg_difs, stddev, sum = 0, max = 0, min = MAX_ITERATIONS;
	int j;

	int iter;
	if (argc != 2) {
		fprintf(stderr, "Usage: %s <number of iterations>\n", argv[0]);
		exit(1);
	}

	/* query for the counter frequency */
	if (QueryPerformanceFrequency(&freq) == 0) {
		fprintf(stderr, "QueryPerformanceFrequency not supported.\n");
		exit(1);
	}

	for (j=0; j<MAX_THRESHOLDS; j++) 
		g_thresh_count[j] = 0;

	/* calculate average difference */
	iter = atoi (argv[1]);
	for (int i=0; i<iter; i++){
		QueryPerformanceCounter(&counter);
		QueryPerformanceCounter(&counter2);
		if (counter2.HighPart != counter.HighPart) {
			fprintf(stderr, "Internal Error\n");
			exit(1);
		}
		dif = counter2.LowPart - counter.LowPart;
		for (j=0; j<MAX_THRESHOLDS; j++)
			if (dif<g_thresh[j]) {
				g_thresh_count[j]++;
				break;
			}

		if (dif > max)
			max = dif;
		if (dif < min)
			min = dif;
		g_results[i] = dif;
		sum_difs += dif;
	}
	avg_difs = (sum_difs / ((float)iter)); //calculate average


	/* calculate stddev */
	for (i=0; i<iter; i++){
		sum += (g_results[i]-avg_difs)*(g_results[i]-avg_difs);
	}
	stddev = sqrt(sum / iter);
	avg_difs *= (1/((float)freq.LowPart))*1000000;   //transform to microseconds
	stddev   *= (1/((float)freq.LowPart))*1000000; 

	fprintf(stderr, "average difference between two successive calls to QueryPerformanceCounter is %3.3f microseconds.\n"
			,avg_difs);
	fprintf(stderr, "maximum time: %3.3f\n", max);
	fprintf(stderr, "minumum time: %3.3f\n", min);
	fprintf(stderr, "Standard Deviation: %3.3f\n", stddev);
	fprintf(stderr, "Histogram :\n");
	for (j=0; j<MAX_THRESHOLDS; j++) {
		if (j>0)
			fprintf (stderr, ">=%d <%d: %d\n", g_thresh[j-1], g_thresh[j], g_thresh_count[j]);
		else
			fprintf (stderr, "< %d : %d\n", g_thresh[j], g_thresh_count[j]);
	}
}

