#include <stdio.h>
#include <time.h>
#include <sys/times.h>
#include <unistd.h>
#include <assert.h>
#include "time_util.h"
#include "i86.h"

#define MAX_ITERATIONS 10000000
#define MAX_THRESHOLDS 14
static double g_results[MAX_ITERATIONS];
int g_thresh[MAX_THRESHOLDS] = { 1000, 2000, 3000, 4000, 5000, 7000, 10000, 30000, 50000, 70000, 10000 , 20000, 50000, 1000000};
int g_thresh_count[MAX_THRESHOLDS];

void main(int argc, char **argv)
{
  struct timespec time1, time2, diff;
  long sum_difs = 0;
  double avg_difs, stddev, sum = 0,max = 0, min = MAX_ITERATIONS;

  clock_t clock1, clock2;

  int iter, i,j;
  if (argc != 2) {
    fprintf(stderr, "Usage: %s <number of iterations>\n", argv[0]);
    exit(1);
  }

  fprintf(stderr, "ticks per sec = %d\n", CLOCKS_PER_SEC);
  /* calculate average difference */
  iter = atoi (argv[1]);
  for (i=0; i<iter; i++){
     clock_gettime( CLOCK_REALTIME, &time1);
     clock_gettime( CLOCK_REALTIME, &time2);
          
     //clock1= clock();
     //clock2 = clock();
     
    diff = t_minus(time2, time1);
     
    assert (diff.tv_nsec >= 0);
    //fprintf(stderr, "%d:%d\n", i, diff.tv_nsec);
    diff.tv_nsec /= 1000;
    //fprintf(stderr, "min clock = %d:%d\n",  diff.tv_sec, diff.tv_nsec);   
   
    for (j=0; j<MAX_THRESHOLDS; j++)
      if (diff.tv_nsec<g_thresh[j]) {
	g_thresh_count[j]++;
	break;
      }
    if (diff.tv_nsec > max)
      max = diff.tv_nsec;
    if (diff.tv_nsec < min)
      min = diff.tv_nsec;
    
    g_results[i] = diff.tv_nsec;
    sum_difs += diff.tv_nsec;
  }
  avg_difs = (sum_difs / ((float)iter)); /* calculate average */
  
  /* calculate stddev */
  for (i=0; i<iter; i++){
    sum += (g_results[i]-avg_difs)*(g_results[i]-avg_difs);
  }
  fprintf(stderr, "sum = %f\n", sum);
  fprintf(stderr, "iter = %d\n", iter);
  stddev = sqrt(sum / (double)iter);
  //stddev1 = (sum / (double)iter);
   
  //fprintf(stderr, "temp = %3.3f\n", stddev1);
  //fprintf(stderr, "sqrt(temp) = %4.4f\n", sqrt(stddev1));
  fprintf(stderr, "average difference between two successive calls to clock() is %3.3f microseconds.\n" ,avg_difs);
  fprintf(stderr, "maximum time: %3.3f\n", max);
  fprintf(stderr, "minumum time: %3.3f\n", min);
  fprintf(stderr, "Standard Deviation: %3.3f\n", stddev);
  fprintf(stderr, "Histogram :\n");
  for (j=0; j<MAX_THRESHOLDS; j++) {
    if (j>0)
      fprintf (stderr, ">=%d <%d: %d\n", g_thresh[j-1], g_thresh[j], g_thresh_count[j]);
    else
      fprintf (stderr, "< %d : %d\n", g_thresh[j], g_thresh_count[j]);
  }
  
  fprintf(stderr, "sqrt(%3.3f / %d) = Standard Deviation: %4.3f\n", sum, iter,stddev);
  clock_getres(CLOCK_REALTIME, &time1);
  fprintf(stderr, "clock resolution: %ld:%3.3f\n (sec:micro)", time1.tv_sec, time1.tv_nsec/1000.0);
}








