#include <stdio.h>
#include <sys/kernel.h>
#include <sys/name.h>
#include <sys/times.h>
#include <sys/types.h>
#include <time.h>
#include "time_util.h"

#define ServerName "/MyServer"
#define MSG_SIZE 128000
#define ITERATIONS 100000

char g_msg[MSG_SIZE], g_rmsg[MSG_SIZE];
  
/*** Prototypes ***/

float measure_bw ( int msg_size, int iterations, pid_t server_pid);

void main()
{
    pid_t server_pid, pid;
    int i; 
    float bw;
    fprintf(stderr, "Locating server: MyServer.\n");
    
    if ((server_pid = qnx_name_locate(0, ServerName, MSG_SIZE, NULL)) < 0) {
    	perror("qnx_name_locate");
    	exit(1);    
    }
 
	fprintf(stderr, "Server located, sending messages...\n");

 	for (i=1; i<2048; i *= 2){	
		bw = measure_bw(i, ITERATIONS, server_pid);
		fprintf(stderr, "Message size = %d; BW = %3.3f\n", i, bw);	   	
	}
}


float measure_bw ( int msg_size, int iterations, pid_t server_pid)
{
	int iter, i;
	pid_t pid;
	struct timespec time_before, time_after, total_time;
	float total_seconds, bw;
	
	clock_gettime(CLOCK_REALTIME, &time_before);
	for (iter=0; iter<iterations; iter++){
		if ((pid = Send(server_pid,  g_msg, g_rmsg, msg_size, 0 )) < 0) {
			perror("Send");
			exit(1);
		}   	
	}
	clock_gettime(CLOCK_REALTIME, &time_after);	
	
	total_time = t_minus(time_after, time_before);
 	total_seconds = total_time.tv_sec + total_time.tv_nsec/1000000000.0;
 	bw = (iterations * msg_size) / total_seconds;
 	 
	fprintf(stderr, "Total seconds: %3.3f\n", total_seconds);
	//fprintf(stderr, "BW = %3.3f bytes/sec\n", bw);

 	return bw;
}



